function initB_EPcont(odefile,sys,point,h,varargin)

%This is an implementation of the orthogonal direction method in order to
%perform branch switching.

%INPUT
%u0 corresponding to f(u0,p) = 0; where p is a vector of paramters and u0
%is a vector of variables. u0 need only be in some neighbourhood of
%attraction of f(u0,p) - i.e such that newton's method will converge. 

%VARARGIN
%opts options for the continuation process, in particular tolerances for
%newton iteration may be useful to adjust. 

global copt;
global curve;


if nargin > 4
    opts = varargin{1};
    setOptions(opts,odefile);
end
if nargin > 5
    disp('Too many input arguments, the last arguments are ignored');
end

x0 = sys.x(:,point);


u0 = x0(1:end-1);
copt.p(copt.ap) = x0(end);
copt.n = length(u0); %dimension of variable space
%copt.k = length(p); %dimension of parameter space
%copt.p  =p;
%copt.ap = ap;

%calculates new tangentvector
Fu0 = calcjac(u0,copt.p);
Fp0 = calcjacp(u0,copt.p,copt.ap);
Fx0 = [Fu0,Fp0];
v0 = (sys.x(:,point) - sys.x(:,point-1));
v0 = v0/psnorm(v0);
%we consider the matrix
A = [Fx0;v0'];
B = A'*A;
[V,D] = eig(B);
mineig = Inf;
pos = 0;
for l = 1:size(D,1)
    val = abs(D(l,l));
    if  val < mineig
        mineig = val;
        pos = l;
    end
end
%the new tangent vector corresponds to the eigenvector belonging to the
%smallest eigenvalue of A*A
v0 = V(:,pos);
v0 = v0/psnorm(v0);
%check that we have the direction of v0 right...
% d = det([Fx0;v0']);
% if  d < 0
%     v0 = -v0;
% end
v0 = copt.orientation*v0;

%sets curve
epcurve = EP;
curve.F = epcurve{1};
curve.Fder = epcurve{2};
curve.testfunctions = epcurve{3};
curve.testlabels = feval(epcurve{4});
curve.process = epcurve{5};
curve.limitcyclecont = false;
curve.locateSingularity = locateBP;

curve.p = copt.p;
curve.ap = copt.ap;
curve.n = copt.n;
curve.k = copt.k;

%predict the next point
xpred = x0 + h*v0;
curve.v0 = v0;
curve.x0 = xpred;
curve.spstart = 1;
end






