function initEP_EPcont(odefile,u0,p,ap,varargin)

%This is an implementation of the orthogonal direction method in order to
%perform branch switching.

%INPUT
%u0 corresponding to f(u0,p) = 0; where p is a vector of paramters and u0
%is a vector of variables. u0 need only be in some neighbourhood of
%attraction of f(u0,p) - i.e such that newton's method will converge. 

%VARARGIN
%opts options for the continuation process, in particular tolerances for
%newton iteration may be useful to adjust. 

global copt;
global curve;


if nargin > 4
    opts = varargin{1};
    setOptions(opts,odefile);
end
if nargin > 5
    disp('Too many input arguments, the last arguments are ignored');
end


copt.n = length(u0); %dimension of variable space
copt.k = length(p); %dimension of parameter space
copt.p  =p;
copt.ap = ap;

%sets curve
epcurve = EP;
curve.F = epcurve{1};
curve.Fder = epcurve{2};
curve.testfunctions = epcurve{3};
curve.testlabels = feval(epcurve{4});
curve.process = epcurve{5};
curve.limitcyclecont = false;
curve.locateSingularity = locateBP;

curve.x0 = [u0;p(ap)];
f = curve.F(curve.x0);

if psnorm(f,2) > copt.tolf
    %then we need to use some newton iterations to get a better
    %approximatiovn
    %corrections
    
    [u0,iter] = newtc(u0,@fval,@fvalder);
    if iter == copt.maxNewtIter
        
        disp('u0 failed to converge to an EP');
        return;
    end
end

curve.x0 = [u0;p(ap)];
%calculates tangentvector 

Fu0 = calcjac(u0,p);
Fp0 = calcjacp(u0,p,copt.ap);
curve.Fx0 = [Fu0,Fp0];
[Q,R] = qr(curve.Fx0');
curve.v0= copt.orientation *det(Q)*det(R(1:end-1,:))*Q(:,end);
curve.v0 = curve.v0/psnorm(curve.v0,2); %tangent vector

curve.p = copt.p;
curve.ap = copt.ap;
curve.n = copt.n;
curve.k = copt.k;
curve.spstart = 0;
end %end initEP

function ansf = fval(u)
global curve;
global copt;
x = [u;copt.p(copt.ap)];
ansf = curve.F(x);
end

function ansf = fvalder(u)
global curve;
global copt;
x = [u;copt.p(copt.ap)];
ansf = curve.Fder(x);
ansf = ansf(:,1:end-1);
end







