function out = MMPDELIB

%--------------------------------------------------------------------------
% This is a library file for GPDEMMPDE.
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%Author: Eirik Ravns - december 2007
%--------------------------------------------------------------------------

%Library functions

out{1} = @G;
out{2} = @mass4;
out{3} = @mass5;
out{4} = @mass6;
out{5} = @mass7;
out{6} = @Cl_G;


end

function ret = Cl_G(t,y,N,funct,v,TAU,smooth,PDE,x0,xNP1)
    u0 = funct{2}(t,v);%boundXL
    uNP1 = funct{3}(t,v);%boundXH
    
    %initializing
    %M0 = -1;
    %M =-1;
    %MNP1 = 1;

    if PDE
        %u = y(1:N);
        x = y(N+1:end);
        f = funct{1}(t,y,v); %F(U,X)
        [M0, M, MNP1] = MonitorPDE(t,y,N,funct,v,x0,xNP1);
    else
        x = y;
        %u = funct{6}(t,x); %evaluating u at (t,x)
        %u0 = funct{2}(t);%boundXL
        %uNP1 = funct{3}(t);%boundXH
        [M0, M, MNP1] = MonitorEx(t,y,N,funct,v,x0,xNP1);
        
    end

    %calculating E(U,X)
    e = zeros(N,1);

    if smooth %smoothing %%%replaces the monitor function by the smoothed analogue
        [M0,M,MNP1] = smoothingg2p2(M0,M,MNP1,N);
    end
    e = -E(x0,x,xNP1,M0,M,MNP1,N);%notice that we don't divide by tau here, in contrast to G
    if PDE
        ret = [f;e];
    else
        ret = e;
    end
end %Cl_G

function ut = G(t,y,N,funct,v,TAU,smooth,PDE,x0,xNP1) %the right hand side of the system

    u0 = funct{2}(t,v);%boundXL
    uNP1 = funct{3}(t,v);%boundXH
    
    %initializing
    %M0 = -1;
    %M =-1;
    %MNP1 = 1;

    if PDE
        %u = y(1:N);
        x = y(N+1:end);
        f = funct{1}(t,y,v); %F(U,X)
        [M0, M, MNP1] = MonitorPDE(t,y,N,funct,v,x0,xNP1);
    else
        x = y;
        %u = funct{6}(t,x); %evaluating u at (t,x)
        %u0 = funct{2}(t);%boundXL
        %uNP1 = funct{3}(t);%boundXH
        [M0, M, MNP1] = MonitorEx(t,y,N,funct,v,x0,xNP1);
        
    end

    %calculating E(U,X)
    e = zeros(N,1);

    if smooth %smoothing %%%replaces the monitor function by the smoothed analogue
        [M0,M,MNP1] = smoothingg2p2(M0,M,MNP1,N);
    end
    e = -E(x0,x,xNP1,M0,M,MNP1,N)/TAU;%notice the minus sign here
    if PDE
        ut = [f;e];
    else
        ut = e;
    end
end %G

function e = E(x0,x,xNP1,M0,M,MNP1,N)
    e = zeros(N,1);
    e(1) = (M(2)+M(1))*(x(2)-x(1))-(M(1)+M0)*(x(1)-x0);
    for i=2:N-1
        e(i) = (M(i+1)+M(i))*(x(i+1)-x(i))-(M(i)+M(i-1))*(x(i)-x(i-1));
    end
    e(N) = (MNP1+M(N))*(xNP1-x(N))-(M(N)+M(N-1))*(x(N)-x(N-1));
    h = 1/(N+1); %length of an interval in the computational domain
    e = 1/(2*h^2)*e;
end %E

function [SM0,SM,SMNP1] = smoothingg2p2(M0,M,MNP1,N)
% Spatial smoothing with gamma = 2, p = 2.
    SM = zeros(N,1);
    for i = 3:N-2
        SM(i) = sqrt((4*M(i-2)^2 + 6*M(i-1)^2 + 9*M(i)^2 + ...
                    6*M(i+1)^2 + 4*M(i+2)^2)/29);
    end
    SM0 = sqrt((9*M0^2 + 6*M(1)^2 + 4*M(2)^2)/19);
    SM(1) = sqrt((6*M0^2 + 9*M(1)^2 + 6*M(2)^2 + 4*M(3)^2)/25);
    SM(2) = sqrt((4*M0^2 + 6*M(1)^2 + 9*M(2)^2 + 6*M(3)^2 + 4*M(4)^2)/29);
    SM(N-1) = sqrt((4*M(N-3)^2 + 6*M(N-2)^2 + 9*M(N-1)^2 + 6*M(N)^2 + 4*MNP1^2)/29);
    SM(N) = sqrt((4*M(N-2)^2 + 6*M(N-1)^2 + 9*M(N)^2 + 6*MNP1^2)/25);
    SMNP1 = sqrt((4*M(N-1)^2 + 6*M(N)^2 + 9*MNP1^2)/19);
end %smoothing

%Evaluating the monitor function at each point
%we use the arclength monitor function (discrete version)
function [M0, M, MNP1] = MonitorPDE(t,y,N,funct,v,x0,xNP1)
    u = y(1:N);
    x = y(N+1:end);
   
    u0 = funct{2}(t,v);%boundXL
    uNP1 = funct{3}(t,v);%boundXH
    
    M = zeros(N,1);
    M0 = sqrt(1+((u(1)-u0)/(x(1)-x0))^2); %this is a discretization of the monitor function
    M(1) = sqrt(1+((u(2)-u0)/(x(2)-x0))^2);
    for i=2:N-1
        M(i) = sqrt(1+((u(i+1)-u(i-1))/(x(i+1)-x(i-1)))^2);
    end
    M(N)= sqrt(1+((uNP1-u(N-1))/(xNP1-x(N-1)))^2);
    MNP1 = sqrt(1+((uNP1-u(N))/(xNP1-x(N)))^2);
end %Mon

function [M0,M,MNP1] = MonitorEx(t,x,N,funct,v,x0,xNP1)
M0 = sqrt(1+funct{1}(t,x0,v)^2);
M = zeros(N,1);
M = sqrt(1+funct{1}(t,x,v).^2);
MNP1=sqrt(1+funct{1}(t,xNP1,v)^2);
end %monitorex

%%Mass Matrices%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%mass matrix for MMPDE4;
function ans4 = mass4(t,y,N,funct,v,PDE,smooth,x0,xNP1)
   
    h = 1/(N+1);
    if PDE
        [M0,M,MNP1] = MonitorPDE(t,y,N,funct,v,x0,xNP1);
        
    else
        [M0,M,MNP1] = MonitorEx(t,y,N,funct,v,x0,xNP1);
        
    end
    if smooth
        [M0,M,MNP1] = smoothingg2p2(M0,M,MNP1,N);
    end
    %this part is common for both cases
    M4 = sparse(N,N);
    M4(1,1) = -(M(2)+2*M(1)+M0);
    M4(1,2) = M(2)+M(1);
    for i=2:(N-1)
        M4(i,i-1) = M(i)+M(i-1);
        M4(i,i) = -(M(i+1)+2*M(i)+M(i-1));
        M4(i,i+1) = M(i+1)+M(i);
    end
    M4(N,N-1) = M(N)+M(N-1); 
    M4(N,N) = -(MNP1+2*M(N)+M(N-1));
    M4= 1/(2*h^2)*M4;
    
    if PDE
        M1 = speye(N);
        M2 = A(t,y,N,funct,v,x0,xNP1);
        M3 = sparse(N,N); %zero matrix
        ans4 =[M1 M2
            M3 M4];
    else %if not PDE
        ans4 = M4;
    end
        
end%mass4


%Mass matrix for MMPDE5
function ans5 = mass5(t,y,N,funct,v,PDE,smooth,x0,xNP1)

    M4 =-1*speye(N);
    if PDE
        M1 = speye(N);
        M2 = A(t,y,N,funct,v,x0,xNP1);
        M3 = sparse(N,N); %zero matrix

        ans5 = [M1 M2
            M3 M4];
        
    else
        ans5 = M4;
    end
end %mass5

%mass matrix for MMPDE6;
function ans6 = mass6(t,y,N,funct,v,PDE,smooth,x0,xNP1)

    e = ones(N,1);
    M4 = spdiags([e -2*e e],-1:1,N,N);
    h = 1/(N+1);
    M4 = 1/h^2*M4;
    if PDE
        M1 = speye(N);
        M2 = A(t,y,N,funct,v,x0,xNP1);
        M3 = sparse(N,N); %zero matrix
        ans6 =[M1 M2
            M3 M4];
    else
        ans6 = M4;
    end
end %mass6

function ans7 = mass7(t,y,N,funct,v,PDE,smooth,x0,xNP1)
    
    h = 1/(N+1);
    if PDE
        [M0,M,MNP1] = MonitorPDE(t,y,N,funct,v,x0,xNP1);
        x = y(N+1:end);
    else
        [M0,M,MNP1] = MonitorEx(t,y,N,funct,v,x0,xNP1);
        x = y;
    end
    if smooth
        [M0,M,MNP1] = smoothingg2p2(M0,M,MNP1,N);
    end
    %this part is common for both cases
    e = E(x0,x,xNP1,M0,M,MNP1,N);
    
    M4 = sparse(N,N);
    ex = 2*e(1)/(x(2)-x0);
    M4(1,1) = - 1/(2*h^2)*(M(2)+2*M(1)+M0);
    M4(1,2) =  1/(2*h^2)*(M(2)+M(1))-ex;
    for i=2:(N-1)
        ex = 2*e(i)/(x(i+1)-x(i-1));
        M4(i,i-1) =  1/(2*h^2)*(M(i)+M(i-1))+ex;
        M4(i,i) = - 1/(2*h^2)*(M(i+1)+2*M(i)+M(i-1));
        M4(i,i+1) =  1/(2*h^2)*(M(i+1)+M(i))-ex;
    end
    ex = 2*e(N)/(xNP1-x(N-1));
    M4(N,N-1) =  1/(2*h^2)*(M(N)+M(N-1))+ex; 
    M4(N,N) = - 1/(2*h^2)*(MNP1+2*M(N)+M(N-1));
     
    
    if PDE
        M1 = speye(N);
        M2 = A(t,y,N,funct,v,x0,xNP1);
        M3 = sparse(N,N); %zero matrix
        ans7 =[M1 M2
            M3 M4];
    else %if not PDE
        ans7 = M4;
    end
        
end%mass7


%help function for mass-matrices       
function ansA = A(t,y,N,funct,v,x0,xNP1)
    
    x = y(N+1:end);
  
    u0 = funct{2}(t,v);%boundXL
    u = y(1:N);
    uNP1 = funct{3}(t,v);%boundXH

    M=sparse(N,N);
        
    M(1,1) = -(u(2)-u0)/(x(2)-x0);
    for i=2:N-1
        M(i,i) = -(u(i+1)-u(i-1))/(x(i+1)-x(i-1));
    end
    M(N,N) = -(uNP1-u(N-1))/(xNP1-x(N-1));
    ansA = M;
end %A




