
function out= LC

% This function specifies the boundary value problem associated with computing an intial limit cycle and its jacobian

out{1} = @f;
out{2} = @fder;
out{3} = [];%@testfunctions;
out{4} = []; %testlabels
out{5} = @adapt;
out{6} = @calcadapt;



end

%% output functions

function ansf = f(x)
global curve;
[u,T,lambda] = discxLC(x);

a = fans(u,T,lambda);
b = fbound(u,T,lambda);
if curve.point == 0;
    c = initfphase(u,T,lambda);
else
    c = fphase(u,T,lambda);
end
ansf = [a;b;c];
end

function ansjac = fder(x)
%ansjac = sparse(jacobian(@f,x));
[u,T,lambda] = discxLC(x);
a = dfder(u,T,lambda);
b = fboundder(u,T,lambda);
c = fphaseder(u,T,lambda);
ansjac = [a;b;c];
ansjac = sparse(ansjac);

end

function newx = adapt(x)
   global copt;
   global curve;
   global adapt;
   
   [uinit,T,lambda] = discxLC(x);
   
   adapt.u = uinit;
   adapt.T = T;
   adapt.lambda = lambda;
   storemesh = curve.maintnodes;
   newmesh = curve.maintnodes;
   iter = 1;
   while iter < copt.maxNewtIter+1
       iter = iter+1;
       newmesh = newmesh(2:end-1);
       g = -adaptfunct(newmesh);
       gx = jacobian(@adaptfunct,newmesh);
       dmesh = gx\g;
       newmesh = [0;newmesh + dmesh;1];
       newfinemesh = settnodes(newmesh);
       adapt.u = getinterpolu(uinit,newfinemesh);
        if iscrossing(newmesh)
            iter = copt.maxNewtIter+1;
            break;
        end
        
        if norm(dmesh,inf) < copt.meshtol
            break;
        end
   end
   if iter > copt.maxNewtIter 
       disp('no convergence of mesh');
       newx = [];
       %bruker gammelt mesh
       %adapt.u = uinit;
       %newmesh = storemesh;
       %newfinemesh = settnodes(newmesh);
       %curve.tnodes = newfinemesh;
       %setGaussPoints();
       %makeLagrangeTables();
       %newx = [adapt.u;T;lambda];
       return
   else
        curve.maintnodes = newmesh;
        newx = [adapt.u;T;lambda];
        curve.tnodes = newfinemesh;
        setGaussPoints();
        makeLagrangeTables();
%         if copt.correctortype == 0 %newton-gauss method
%             corrector = @fastcorr2;
%         elseif copt.correctortype == 1 %pseudo-arclength
%             corrector = @fastcorr3;
%         elseif copt.correctortype == 2 %quasi-pseudo-arclength
%             corrector = @fastcorr;
%         elseif copt.correctortype == 3 %conjugate gradient
%             corrector = @CGcorr;
%         end
%         [nx,v,iter] = corrector(newx,@f,@fder);
%         if iter > copt.maxNewtIter || isempty(nx)
%             disp('no convergence of mesh 2');
%             newx = [];
%             return;
%         end
%         newx = nx;
   end
end

function a =  adaptfunct(newmesh)
       global adapt;
       %global copt;
       u = adapt.u;
       T = adapt.T;
       lambda = adapt.lambda;
       om0 = 0;
       om = newmesh;
       omNP1 = 1;
       a = E(u,T,lambda,om0,om,omNP1);       
end

function b = iscrossing(newmesh)

val = -0.1;
b = false;
    for i=1:length(newmesh)
        if newmesh(i) <= val
            disp('mesh crossing');
            b = true;
            return;      
        else
            val = newmesh(i);
        end
    end
end
%% collocation functions
function ansf = fans(u,T,lambda)
global copt;
%global curve;
%ansf = zeros(copt.ntst*copt.ncol*copt.dim + nq + 1,1);
ansf = zeros(copt.ntst*copt.ncol*copt.n,1);
%nn = copt.ntst*copt.ncol;
p = copt.p;
p(copt.ap) = lambda;
counter = 1;
    for j=1:copt.ntst
    
        for i= 1:copt.ncol
            %t = curve.gauss(j,i);           
            %F = polyder(j,t,u) - T*copt.F(poly(j,t,u),p);
            F = polyder(j,i,u) - T*copt.F(poly(j,i,u),p);
            c1 = counter;
            c2 = counter+copt.n-1;
            ansf(c1:c2) = F;
            counter = counter + copt.n;
        end
    end
end

%this is useful for calculating jacobians
function F = smallf(j,i,u,T,lambda)
    global copt;
    p = copt.p;
    p(copt.ap) = lambda;
    F = polyder(j,i,u) - T*copt.F(poly(j,i,u),p);
end

function Jacu = dfder(u,T,lambda)
global copt;
Jacu = [];
for j=1:copt.ntst
    for i=1:copt.ncol
        jac = df(j,i,u,T,lambda);
      
        if isempty(Jacu) 
            Jacu= jac;
        else
            Jacu = [Jacu; jac];
        end
    end
end
end %dfder

function jac = df(j,i,u,T,lambda)
global copt;
dim = length(u);
jac = sparse(copt.n,dim+2);
jac(:,end-1) = pder2(@smallf,j,i,u,T,lambda,dim+1);
jac(:,end) = pder2(@smallf,j,i,u,T,lambda,dim+2);

    for q = 1:copt.ncol+1
        for l = 1:copt.n
            index = copt.contbl(j,q,l);
            df = pder2(@smallf,j,i,u,T,lambda,index);
            jac(:,index) = df;
        end
    end
 
end

%% boundary condition

function ansfb = fbound(u,T,lambda)
global copt;
u1 = getelem(u,1,1);
u2 = getelem(u,copt.ntst,copt.ncol+1);
ansfb = u1- u2;
end

function ansfb = fboundder(u,T,lambda)
global copt;
dim = length(u);
ansfb = [eye(copt.n), zeros(copt.n,dim-2*copt.n), -eye(copt.n),zeros(copt.n,2)];
end

%% phase functions

function sum = fphase(u,T,lambda)
global copt;
global curve;

ncweights = ncw(copt.ncol); %obs m \in [2,7]
sum = 0;
%if ~copt.adaptmesh
[u0,T0,lambda0] = discxLC(curve.x0);
p0 = copt.p;
p0(copt.ap) = lambda0; 
%else
%    [u0,T0,lambda0] = discxLCAdapt(curve.x0);
%end
for j=1:copt.ntst
    for i=1:copt.ncol + 1
        uk = getelem(u,j,i);%,copt.ncol,copt.n);
        uu0 = getelem(u0,j,i);
        dotu0 = T0*copt.F(uu0,p0);%,copt.ncol,copt.n);
        q = uk'*dotu0;
        sum = sum+ncweights(i)*q;
    end
end
end

function sum = initfphase(u,T,lambda)
global copt;
%global curve;

ncweights = ncw(copt.ncol); %obs m \in [2,7]
sum = 0;
%[u0,T0,lambda0] = discxLC(curve.x0);
%p0 = copt.p;
%p0(copt.ap) = lambda0; 

%[u0,T0,lambda0] = discxLC(curve.x0);
u0 = copt.phider;
for j=1:copt.ntst
    for i=1:copt.ncol + 1
        uk = getelem(u,j,i);%,copt.ncol,copt.n);
        dotu0 = getelem(u0,j,i);
        q = uk'*dotu0;
        sum = sum+ncweights(i)*q;
    end
end
end

function jac = fphaseder(u,T,lambda)
global curve;
dim = length(u)+2;
jac = zeros(1,dim);
if curve.point > 0
    for i=1:length(jac);
        jac(i) = pder(@fphase,u,T,lambda,i);
    end
else
    for i=1:length(jac);
        jac(i) = pder(@initfphase,u,T,lambda,i);
    end
end
end

%% support functions
function sum = poly(j,i,u,varargin)
global copt;
global curve;
%note that j,i is used for indexing z_j,i
%j = interval;
sum = 0;
for k=1:copt.ncol+1
    %l = lagrange(j,i,t);
    l = curve.gausslag(j,i,k); 
    sum = sum + l*getelem(u,j,k);
end
end

function sum = interpoly(j,t,u)
global copt;
sum = 0;
for i=1:copt.ncol+1
    sum = sum + lagrange(j,i,t)*getelem(u,j,i);
end
end

function sum = polyder(j,i,u)
global copt;
global curve;
sum = 0;
for k=1:copt.ncol+1
    
    %l = lagrangeder(j,i,t);
    l = curve.gausslagder(j,i,k);
    sum = sum + l*getelem(u,j,k);%,copt.ncol,copt.n);
end
end

function [u,T,lambda] = discxLC(x)
global copt;
global curve;
dim = length(curve.tnodes)*copt.n;
u = x(1:dim);
lambda = x(end);
T = x(end-1);
end

function j = pder(funct,u,T,lambda,index)
global copt;
dim = length(u);
if index == dim+1
    T1 = T -copt.jacincr;
    T2 = T + copt.jacincr;
    j  = funct(u,T2,lambda) - funct(u,T1,lambda);
elseif index == dim+2
    lambda1 = lambda - copt.jacincr;
    lambda2 = lambda + copt.jacincr;
    j = funct(u,T,lambda2) - funct(u,T,lambda1);
elseif index <= dim
x = u;
x1 = x; 
x1(index) = x1(index)-copt.jacincr;
x2 = x; 
x2(index) = x2(index)+copt.jacincr;
j = funct(x2,T,lambda) - funct(x1,T,lambda);
else
    disp('wrong index given in pder');
    j = [];
    return;
end
j = j/(2*copt.jacincr);
end

function jac = pder2(funct,j,i,x,T,lambda,index)
global copt;
dim = length(x);
if index == dim+1
    T1 = T -copt.jacincr;
    T2 = T + copt.jacincr;
    jac  = funct(j,i,x,T2,lambda) - funct(j,i,x,T1,lambda);
elseif index == dim+2
    lambda1 = lambda - copt.jacincr;
    lambda2 = lambda + copt.jacincr;
    jac = funct(j,i,x,T,lambda2) - funct(j,i,x,T,lambda1);
elseif index <= dim
x1 = x; 
x1(index) = x1(index)-copt.jacincr;
x2 = x; 
x2(index) = x2(index)+copt.jacincr;
jac = funct(j,i,x2,T,lambda) - funct(j,i,x1,T,lambda);
else
    disp('wrong index given in pder2');
    jac = [];
    return;
end
jac = jac/(2*copt.jacincr);
end

%%


function w = getinterpolu(u,newtnodes)
    global copt; 
    w = zeros(size(u));
     counter = 1;
     for j=1:copt.ntst
         for i=1:copt.ncol
             w(counter:counter+copt.n-1) = interpolate(j,i,u,newtnodes);
             counter = counter + copt.n; 
         end
     end
     w(counter:counter+copt.n-1) = interpolate(copt.ntst,copt.ncol+1,u,newtnodes);
end

function mainu = getmainu(u)
    global curve;
    global copt;
    
    mainu = zeros(length(curve.maintnodes)*copt.n,1);
    counter = 1;
    for j=1:copt.ntst
        mainu(counter:counter+copt.n-1) = getelem(u,j,1);
        counter = counter + copt.n;
    end
    mainu(counter:end) = getelem(u,copt.ntst,copt.ncol+1);
end

function newu = interpolate(j,i,u,newtnodes)
global copt;
global curve;
  %interpolate oldu onto the new mesh
ind = getind(j,i,copt.ncol);
t = newtnodes(ind);

for l=2:length(curve.maintnodes) %i.e old main nodes
    if t < curve.maintnodes(l)
       %then we use p_l for interpolation
       newu = interpoly(l-1,t,u);
       return;
    end
end

newu = interpoly(l-1,t,u);
end

%% functions for mesh adaptation

function e = E(u,T,lambda,x0,x,xNP1)
    global copt;
    %global curve;
    mon = monitor(u,T,lambda);
    M0 = mon(1);
    MNP1 = mon(end);
    M = mon(2:end-1);
    N = copt.ntst-1; %the number of internal nodes
    if copt.smoothing %smoothing %%%replaces the monitor function by the smoothed analogue
        [M0,M,MNP1] = smoothingg2p2(M0,M,MNP1,N);
    end
    
    e = zeros(N,1);
    e(1) = (M(2)+M(1))*(x(2)-x(1))-(M(1)+M0)*(x(1)-x0);
    for i=2:N-1
        e(i) = (M(i+1)+M(i))*(x(i+1)-x(i))-(M(i)+M(i-1))*(x(i)-x(i-1));
    end
    e(N) = (MNP1+M(N))*(xNP1-x(N))-(M(N)+M(N-1))*(x(N)-x(N-1));
    %h = 1/(N+1); %length of an interval in the computational domain
    %e = 1/(2*h^2)*e;
end %E


function mon= monitor(u,T,lambda)
global copt;
global curve;
mon = zeros(size(curve.maintnodes));
counter = 1;
p = copt.p;
p(copt.ap) = lambda;
if curve.point > 0
    u = getmainu(u);
    for j=1:length(mon)
        up = u(counter:counter+copt.n-1);
        counter = counter + copt.n;
        mon(j) = norm(T*copt.F(up,p));
    end
else
    uder = getmainu(copt.phider);
    for j=1:length(mon)
        up = uder(counter:counter+copt.n -1);
        counter = counter + copt.n;
        mon(j) = norm(up); 
    end
end
end %monitor

function [SM0,SM,SMNP1] = smoothingg2p2(M0,M,MNP1,N)
% Spatial smoothing with gamma = 2, p = 2.
    SM = zeros(N,1);
    for i = 3:N-2
        SM(i) = sqrt((4*M(i-2)^2 + 6*M(i-1)^2 + 9*M(i)^2 + ...
                    6*M(i+1)^2 + 4*M(i+2)^2)/29);
    end
    SM0 = sqrt((9*M0^2 + 6*M(1)^2 + 4*M(2)^2)/19);
    SM(1) = sqrt((6*M0^2 + 9*M(1)^2 + 6*M(2)^2 + 4*M(3)^2)/25);
    SM(2) = sqrt((4*M0^2 + 6*M(1)^2 + 9*M(2)^2 + 6*M(3)^2 + 4*M(4)^2)/29);
    SM(N-1) = sqrt((4*M(N-3)^2 + 6*M(N-2)^2 + 9*M(N-1)^2 + 6*M(N)^2 + 4*MNP1^2)/29);
    SM(N) = sqrt((4*M(N-2)^2 + 6*M(N-1)^2 + 9*M(N)^2 + 6*MNP1^2)/25);
    SMNP1 = sqrt((4*M(N-1)^2 + 6*M(N)^2 + 9*MNP1^2)/19);
end %smoothing

