function initH_LCcont(odefile,sys,point,amp,varargin)

%This function initializes limit cycle continuation from a hopf point (u0,p) and
%calculates a direction vector if it is not already provided in the input
%arguments. 

%INPUT
%u0 corresponding to f(u0,p) = 0; where p is a vector of paramters and u0
%is a vector of variables. u0 need only be in some neighbourhood of
%attraction of f(u0,p) - i.e such that newton's method will converge. 
%VARARGIN
%opts options for the continuation process, in particular tolerances for
%newton iteration may be useful to adjust. 

global copt;
global curve;

maintnodes = [];
if nargin > 4
    opts = varargin{1};
    setOptions(opts,odefile);
end
if nargin > 5
    maintnodes = varargin{2};
end
if nargin > 6
    disp('Too many input arguments, the last arguments are ignored');
end

%sets curve
lccurve = LC;
curve.F = lccurve{1};
curve.Fder = lccurve{2};
curve.testfunctions = lccurve{3};
curve.testlabels = lccurve{4};
curve.limitcyclecont = true;
curve.adapt = lccurve{5};
curve.initadapt = lccurve{6};

x0 = sys.x(:,point);
u0 = x0(1:end-1);
copt.p(copt.ap) = x0(end);
p = copt.p;
Fu0 = calcjac(u0,p);

%find the eigenvalues that makes this a hopf bifurcation point
[V,D] = eig(Fu0);
eigvals = diag(D);
minval = Inf;
indicies = zeros(2,1);
n = length(eigvals);
for j=1:n-1
  [val,ind] = min(abs(eigvals(j+1:n)+eigvals(j)));
  if val < minval
    indicies(1) = j;
    indicies(2) = j+ind;
    minval = val;
  end
end
%test if we really are at a bifurcation point
if imag(eigvals(indicies(1))) == 0 && imag(eigvals(indicies(2))) == 0
  disp('error: Neutral saddle\n');
  return;
end

%
ceigvec = V(:,indicies(1));
wr = real(ceigvec);
wi = imag(ceigvec);
freq = abs(imag(eigvals(indicies(1))));  
T0 = 2*pi/freq;

if isempty(maintnodes)
    maintnodes = 0:1/copt.ntst:1;%initially the mesh is uniform
    
end
maintnodes = maintnodes';
%nint = copt.ntst*(copt.ncol);
%tint = 1/nint;
%tnodes = 0:tint:1;
%curve.tnodes = tnodes';
%maintnodes = 0:1/copt.ntst:1;%initially the mesh is uniform
tnodes = settnodes(maintnodes);
curve.tnodes = tnodes;
maintnodes = maintnodes(2:end-1);
setGaussPoints();
makeLagrangeTables();

phiv = zeros(copt.n,length(tnodes));
phider = zeros(copt.n,length(tnodes));
for i=1:length(tnodes)
     t = tnodes(i);
     phiv(:,i) = wr*cos(2*pi*t)-wi*sin(2*pi*t);
     phider(:,i) = 2*pi*(-wr*sin(T0*freq*t)-wi*cos(T0*freq*t));
     
end
dim = size(phiv);
phivals = reshape(phiv,dim(1)*dim(2),1);
copt.phider = reshape(phider,dim(1)*dim(2),1);
phivals = phivals/norm(phivals); %normalizing
copt.phider = copt.phider/norm(copt.phider);
copt.phivals = phivals;

%creating a conversion table
copt.contbl = zeros(copt.ntst,copt.ncol+1,copt.n);
counter = copt.n+1;
for j=1:copt.ntst
    for i= 1:copt.ncol+1
       for k=1:copt.n
            if i==1 && k==1
                counter = counter -copt.n;
            end
            copt.contbl(j,i,k) = counter;
            counter = counter+1;
        end
    end
end

% if ~copt.adaptmesh
    curve.v0 = [phivals;0;0];
    curve.n = length(curve.v0)-1;
    curve.x0 = repmat(u0,copt.ntst*copt.ncol +1,1);
    curve.x0 = [curve.x0 + amp*phivals;T0;copt.p(copt.ap)];
% else
%     curve.v0 = [phivals;0;zeros(size(maintnodes));0];
%     curve.n = length(curve.v0)-1;
%     curve.x0 = repmat(u0,copt.ntst*copt.ncol+1,1);
%     curve.x0 = [curve.x0 + amp*phivals;T0;maintnodes;copt.p(copt.ap)];
% 
% end
curve.maintnodes = [0;maintnodes;1];
curve.spstart = 1;

end %init


    

