function out = continuer(varargin)
global out;
%This function is the continuator.
%The function requires that an intializer function such as initEP_EPcont or
%initH_LC is called first in order to set the required variables.

if nargin > 0 && nargin > 1
    odefile = varargin{1};
    opts = varargin{2};
    setOptions(opts,odefile);
elseif nargin > 0
    disp('Wrong number of input arguments');
    out = [];
    return;
end

if nargin > 2
    disp('Too many input arguments, the last arguments are ignored');
end

global copt;
global curve;

%initializing which procedures shall be used 
adaptsteplength = @simpleadaptstep;
funct = curve.F;
functder = curve.Fder;
if copt.secanttangent
    tangentvec = @getsecanttangent;
else
    tangentvec = @gettangent;
end
if copt.correctortype == 0 %newton-gauss method
    corrector = @fastcorr2;
elseif copt.correctortype == 1 %pseudo-arclength
    corrector = @fastcorr3;
elseif copt.correctortype == 2 %quasi-pseudo-arclength
    corrector = @fastcorr;
else
   disp('invalid corrector chosen');
   out = [];
   return;
end

%output
out = [];
out.x = zeros(length(curve.x0),copt.numPoints);
out.v = zeros(length(curve.x0),copt.numPoints);
if copt.detectBifurcationPoints
    out.f = zeros(length(curve.x0)-1,copt.numPoints);
else
    out.f = [];
end
if copt.adaptmesh
    out.mesh = zeros(length(curve.tnodes),copt.numPoints);
    
end

curve.point = 1;
if curve.spstart
    curve.point = 0;
end  


disp('starting continuer');
starttime = clock();
Fx0 = curve.Fder(curve.x0);
Fu0 = Fx0(1:curve.n,1:curve.n);

if copt.detectBifurcationPoints
    eig1 = eig(full(Fu0));
    stableeigs1 = countstableeigs(eig1);
    testvals1 = [];
    testvals0 = []; 
end
if ~curve.spstart
out.x(:,1) = curve.x0;
    out.v(:,1) = curve.v0;
    
    if copt.detectBifurcationPoints
        out.f(:,1) = eig1;
    %elseif copt.adaptmesh
    %    curve.settnodes();
    end
end

out.s.index = [1];
out.s.label = ['s'];
coeff = 1;
bifIter = 0;
detected = 0;
found = false;
id =0;
minstep = false;
x1 = curve.x0;
v1 = curve.v0;
Fx1 = Fx0;

while curve.point < copt.numPoints 

    if copt.printprogress
        disp(['approximating the point ' num2str(curve.point+1)  ' ap = ' num2str(x1(end))]);
    end
    curve.x0 = x1;
    curve.v0 = v1;
    Fx0 = Fx1;
    if copt.detectBifurcationPoints
        eig0 = eig1;
        stableeigs0 = stableeigs1;
        testvals0 = testvals1;
    end

    %prediction
  
        x1 = curve.x0 + copt.h * curve.v0;
        
    
    %corrections
    [x1,v1,iter] = corrector(x1,funct,functder);
   
    if isempty(x1)
        iter = copt.maxNewtIter+1; %i.e no convergence
        %x1 = curve.x0;
    end
    if iter < copt.maxNewtIter && curve.limitcyclecont && copt.adaptmesh 
       if curve.point > copt.startadapt 
          x1 = curve.adapt(x1);
          if isempty(x1)
             iter = copt.maxNewtIter +1;
          end
       end
    end
    
    if  iter > copt.maxNewtIter
        %no convergence: 
        %we use the previous point...
         x1 = curve.x0;
         v1 = curve.v0;
         Fx1 = Fx0;
         if copt.detectBifurcationPoints
            eig1 = eig0;
            stableeigs1 = stableeigs0;
            testvals1 = testvals0;
         end

         if detected == 1
             bifIter = bifIter +1;
         end
    else
        %convergence
            %a new point may have been found 

            Fx1 = curve.Fder(x1);
            Fu1 = Fx1(:,1:curve.n);
            %tangent vector
            if isempty(v1)
                v1 = tangentvec(Fx1,curve.x0,x1,copt.h);
            end
            if copt.detectBifurcationPoints;
                eig1 = eig(full(Fu1));
                stableeigs1 = countstableeigs(eig1);
                diff = abs(stableeigs1-stableeigs0);
                
                if ~isempty(curve.testfunctions) && curve.point > 1
                    if ~detected && diff ~= 0
                        if diff > 2
                            disp('High singularity may have been found');
                            detected = 2;
                        end
                      
                        %we must check what type of bifurcation point has
                        %occured
                        testvals0 = curve.testfunctions(Fx0,curve.v0,eig0);%evaluate last point
                        testvals1 = curve.testfunctions(Fx1,v1,eig1);
                        coeffs = testvals0.*testvals1;
                        id = find(coeffs<=0);
                        if isequal(id,[1;3]);
                            id = 3; 
                        end
                        coeff = -testvals1(id)/(testvals1(id)-testvals0(id));
                      
                        if length(coeff) > 1 || detected == 2;
                           detected = 2;
                       
                        else
                            detected = 1;
                            bifIter = 1;
                            label = curve.testlabels(id);
                            disp([label ' detected at ' num2str(curve.point+1)]);
                            %store the variables for the previous point
                            x2 = x1;
                            v2 = v1;
                            eig2 = eig1;
                            Fx2 = Fx1;
                            h2 = copt.h;
                            stableeigs2 = stableeigs1;
                            testvals2 = testvals1;
                        end
                    elseif detected == 1
                         %we must evaluate testfunctions to obtain a
                         %new steplength coefficient

                         testvals1 = curve.testfunctions(Fx1,v1,eig1);
                         coeffs = testvals0.*testvals1;
                         id2 = find(coeffs<0);
                         if isequal(id2,[1;3]);
                            id2 = 3; 
                         end
                         coeff =-testvals1(id)/(testvals1(id)-testvals0(id));
                         if length(id2) > 1 || (length(id2)== 1 && ~isequal(id2,id))
                            disp('warning: stepsize to big, multiple bifurcation points passed');
                            detected = 2;
                         end
                             
                    end
                    
                end       
            end

            %check if point is accepted       

            if detected == 1
                bifIter = bifIter +1;
                mineig = min(abs(real(eig1)));
                if abs(mineig) < copt.testtol
                   %bifurcation point accepted
                   found = true;
                elseif label == 'B' && copt.singularityLocator; 
                    
                    [found,x1] = locSing(x1);
                    if ~found
                        disp('singularity not found, aborting continuation process');
                        out = fixoutput(curve.point,out);
                        return;
                    end
                    Fx1 = curve.Fder(x1);
                    Fu1 = Fx1(:,1:curve.n); 
                    eig1 = eig(full(Fu1));
                    v1 = curve.v0; %using the previous tangent vector as an approximate value
                    testvals1 = testvals0;
                end
            end                
            %new point=?
            
                
            if  detected == 0       
                curve.point = curve.point+1;
               
                out.x(:,curve.point) = x1; % new point found
                
                if copt.detectBifurcationPoints
                    out.f(:,curve.point) = eig1;
                elseif copt.adaptmesh
                    out.mesh(:,curve.point) = curve.tnodes;
                end
                out.v(:,curve.point) = v1;
                
            elseif detected == 2
                    %we use the previous point...
                    x1 = curve.x0;
                    v1 = curve.v0;
                    eig1 = eig0;
                    stableeigs1 = stableeigs0;
                    Fx1 = Fx0;
                    detected = 0;
                                        
            else
                if found
                    %if a new special point is accepted, we update the structure
                    %out.s
                    bifIter = 0;
                    found = false;
                    detected = 0;
                    id = 0;
                    %testvals0 = [];
                    %testvals1 = [];
                                        
                    curve.point = curve.point+1;
                    out.x(:,curve.point) = x1; % new point found
                    out.f(:,curve.point) = eig1;
                    out.v(:,curve.point) = v1;
                    out.s.index = [out.s.index curve.point];
                    out.s.label = [out.s.label label]; 
                    copt.h = h2; %go back to previous steplength
                  
                    if ~isequal(x1,x2)
                    %we have already found the next point
                    x1 = x2;
                    eig1 = eig2;
                    v1 = v2;
                    stableeigs1 = stableeigs2;
                    Fx1 = Fx2;
                    testvals1 = testvals2;
                    curve.point = curve.point+1;
                    out.x(:,curve.point) = x2; % new point found
                    out.f(:,curve.point) = eig2;
                    out.v(:,curve.point) = v2;
                                  
                    end
                end
                        
            end   
                   
    end
   
        %steplength adaption
        [minstep,quit] = adaptsteplength(minstep,detected,found,iter,bifIter,coeff);
        if quit
            endtime = clock();
            disp(['continuation halted: time elapsed ' num2str(etime(endtime, starttime)) ' seconds'] );
            out = fixoutput(curve.point,out);
            return;
        end
end

endtime = clock();
disp(['continuation finished: time elapsed ' num2str(etime(endtime, starttime)) ' seconds'] );
out = fixoutput(curve.point,out);
end %continuer

function fixout =  fixoutput(point, out)
global copt;
fixout.x = out.x(:,1:point);
fixout.v = out.v(:,1:point);
if copt.detectBifurcationPoints
    fixout.f = out.f(:,1:point);
end
if copt.adaptmesh
    fixout.mesh = out.mesh(:,1:point);
end
fixout.s.index = [out.s.index point];
fixout.s.label = [out.s.label 'e'];
end

function [found,x] = locSing(x)
   global curve;
   global copt;
   corrector = @newtc;
   %modifying the tolerance
   oldtolx = copt.tolxinf;
   oldtolf = copt.tolfinf;
   accvec = [0,0,1,1,0,0];
   copt.tolxinf = copt.testsing;
   copt.tolfinf = copt.testsing;
   found = false;
   init = curve.locateSingularity{1};
   y = init(x);
   f = curve.locateSingularity{2};
   fder = curve.locateSingularity{3};
   [y1,v,iter] = corrector(y,f,fder,copt.newteval,accvec);
   if iter < copt.maxNewtIter && ~isempty(y1)
       found = true;
       y = y1;
 
   end
   copt.tolxinf = oldtolx;
   copt.tolfinf = oldtolf;
   x = y(1:copt.n+1);
end


