%this is a template for how to create F structures to go along with
%GPDEMMPDE
function out = gspike

out{1} = @F;%
out{2} = @boundXL;
out{3} = @boundXH;
out{4} = @boundT0;
out{5} = @pde;
out{6} = @exact;
out{7} = @initGrid;
out{8} = @initParam;

%CASE 1: the function corresponds to the discrete form of the right hand side of a
%PDE of the form u_t = F(u,u_x,U_xx,...)
%INPUT: a time t, and a vector y = [u,x]:
%OUTPUT: F(t,y) (vector)
%CASE 2: the function evaluates u_x(t,x)
function ut = F(t,y,v) 
x = y; %since we know the exact solution, only x'es are calculated
a = v(1);
epsilon = v(2);

ut = zeros(size(x));
for i=1:length(ut)
    ut(i) = -(x(i)-a)/epsilon^2*t.*exp(-(x(i)-a).^2/(2*epsilon^2));%den deriverte av u m.h.p x
end

%boundary conditions: u(xmin,t) =
function ansXL= boundXL(t,v)
a = v(1);
epsilon = v(2);
xmin = 0;
ansXL = t*exp(-(xmin-a)^2/(2*epsilon^2));

%u(xmax,t) =
function ansXH = boundXH(t,v)
a = v(1);
epsilon = v(2);
xmax = 1;
ansXH = t*exp(-(xmax-a)^2/(2*epsilon^2));

%initial condition, u(x,0)=
function ans = boundT0(x,v)
ans = 0;

%return 1 whenever the exact solution is known, otherwise return 0;
%if known, the exact solution is written in the F function
function anspde = pde()
anspde = 0;

%u(t,x)
function ansex = exact(t,y,v)
x = y; %since we know the exact solution, only x'es are calculated
a = v(1);
epsilon = v(2);
ansex = zeros(size(x));
for i=1:length(ansex)
   ansex(i) = t*exp(-(x(i)-a)^2/(2*epsilon^2));
end

%if a function for specifing init grid is defined. For ansinit = x, the
%same initial grid as for the computational domain is used
function ansinit = initGrid(x,v)
ansinit = x;
return

function v = initParam()
epsilon = 0.007;
a = 0.30015;
v = [a,epsilon];



