%this is a template for how to create F structures to go along with
%GPDEMMPDE
function out = gspikes3

out{1} = @F;%
out{2} = @boundXL;
out{3} = @boundXH;
out{4} = @boundT0;
out{5} = @pde;
out{6} = @exact;
out{7} = @initGrid;
out{8} = @initParam;

%CASE 1: the function corresponds to the discrete form of the right hand side of a
%PDE of the form u_t = F(u,u_x,U_xx,...)
%INPUT: a time t, and a vector y = [u,x]:
%OUTPUT: F(t,y) (vector)
%CASE 2: the function evaluates u_x(t,x)
function ut = F(t,y,v) 
x = y; %since we know the exact solution, only x'es are calculated
a1 = v(1);
a2 = v(2);
e1 = v(3);%epsilon1
e2 = v(4);%epsilon2

ut = zeros(size(x));
for i=1:length(ut)
    ut(i) =-t*(x(i)-(a1+t/2))/e1^2*exp(-1/2*(x(i)-(a1+t/2))^2/e1^2)-t*(x(i)-(a2-t/2))/e2^2*exp(-1/2*(x(i)-(a2-t/2))^2/e2^2);
end

%boundary conditions: u(xmin,t) =
function ansXL= boundXL(t,v)
ansXL = exact(t,0,v);


%u(xmax,t) =
function ansXH = boundXH(t,v)
ansXH = exact(t,1,v);

%initial condition, u(x,0)=
function ansT = boundT0(x,v)
ansT = exact(0,x,v);

%return 1 whenever the exact solution is known, otherwise return 0;
%if known, the exact solution is written in the F function
function anspde = pde()
anspde = 0;

%u(t,x)
function ansex = exact(t,y,v)
x = y; %since we know the exact solution, only x'es are calculated
a1 = v(1);
a2 = v(2);
e1 = v(3);%epsilon1
e2 = v(4);%epsilon2

ansex = zeros(size(x));
for i=1:length(ansex)
    g1  = t*exp(-1/2*(x(i)-(a1+t/2))^2/e1^2);
    g2  = t*exp(-1/2*(x(i)-(a2-t/2))^2/e2^2);
    ansex(i) = g1+g2;
end


%if a function for specifing init grid is defined. For ansinit = x, the
%same initial grid as for the computational domain is used
function ansinit = initGrid(x,v)
ansinit = x;
return

function v = initParam()
e1 = 0.007;
e2 = 0.007;
a1 = 0.5;
a2 = 0.5;
v = [a1,a2,e1,e2];



