function plotsys(x,v,s,varargin)
%
% plots computed curve: cpl(x,v,s)
% dim(x) = 2 or 3
%
%grid on;
f = [];
if nargin > 3
  x = x(varargin{1},:);
  v = v(varargin{1},:);
  if nargin > 4
          f = varargin{2};
  end
end
[dim,stop]=size(x);

watchon;
switch dim
  case 2
      if isempty(f)
          plot2d(x,v,s);
      else
        plot2dstab(x,v,f,s);
      end
  case 3
      if isempty(f)
          plot3d(x,v,s);
      else
        plot3dstab(x,v,f,s);
      end
    
  otherwise
    error('This dimension is not supported');
end
watchoff;
ax=get(gcf,'CurrentAxes');
set(ax,'FontSize',10);
%axis([0 1 0 1]);
%axis([0 3 minX maxX]);


function plot2d(x,v,s)
global copt;
points = cat(1,s.index); %converts structure to a vector of indicies
line(x(1,:),x(2,:),'linestyle',copt.plotstable,'color',copt.plotcolor);
points = points(2:end-1);
line(x(1,points),x(2,points),'linestyle','none','marker',copt.plotmarker,'color',copt.plotcolormark,'LineWidth',copt.plotlinewidth);
d=axis;
skew = 0.03*[d(2)-d(1) d(4)-d(3)];
labels = cat(1,s.label);
for i=2:length(s.index)-1;
text(x(1,s.index(i))+skew(1),x(2,s.index(i))+skew(2),labels(i));
end


function plot2dstab(x,v,f,s)
global copt;
%[dim, m] = size(x);
points = cat(1,s.index); %converts structure to a vector of indicies
hold on;
for i=2:(length(points))
    checkpoint = round((points(i)+points(i-1))/2);%select a point between two markers
    isunstable = 0;
    for j=1:length(f(:,checkpoint))
        if real(f(j,checkpoint)) > 0
            isunstable = 1;
        end
    end
     if isunstable
        %draw dotted line
        line(x(1,points(i-1):points(i)),x(2,points(i-1):points(i)),'linestyle',copt.plotunstable,'color',copt.plotcolor,'LineWidth',copt.plotlinewidth);
    else
        %draw full line
        line(x(1,points(i-1):points(i)),x(2,points(i-1):points(i)),'linestyle',copt.plotstable,'color',copt.plotcolor,'LineWidth',copt.plotlinewidth);
    end
end
hold off;
%line(x(1,:),x(2,:),'linestyle','-','color','black');
points = points(2:end-1);
line(x(1,points),x(2,points),'linestyle','none','marker',copt.plotmarker,'color',copt.plotcolormark);
d=axis;
skew = 0.03*[d(2)-d(1) d(4)-d(3)];
labels = cat(1,s.label);
for i=2:length(s.index)-1;
text(x(1,s.index(i))+skew(1),x(2,s.index(i))+skew(2),labels(i));
end

function plot3d(x,v,s)
global copt;
view(3);
line(x(1,:),x(2,:),x(3,:),'linestyle',copt.plotstable,'color',copt.plotcolor,'LineWidth',copt.plotlinewidth);
points = cat(1,s.index); %converts structure to a vector of indicies
points = points(2:end-1);
line(x(1,points),x(2,cat(1,points)),x(3,points),'linestyle','none','marker',copt.plotmarker,'color',copt.plotcolormark); %*
d=axis;
skew = 0.03*[d(2)-d(1) d(4)-d(3) d(6)-d(5)];
labels = cat(1,s.label);
for i=2:length(s.index)-1;
text(x(1,s.index(i))+skew(1),x(2,s.index(i))+skew(2), x(3,s.index(i))+skew(3),labels(i));
end

function plot3dstab(x,v,f,s)
global copt;
view(3);
%line(x(1,:),x(2,:),x(3,:),'linestyle','-','color','k');
%[dim, m] = size(x);
points = cat(1,s.index); %converts structure to a vector of indicies
hold on;
for i=2:(length(points))
    checkpoint = round((points(i)+points(i-1))/2);%select a point between two markers
    isunstable = 0;
    for j=1:length(f(:,checkpoint))
        if real(f(j,checkpoint)) > 0
            isunstable = 1;
        end
    end
     if isunstable
        %draw dotted line
        line(x(1,points(i-1):points(i)),x(2,points(i-1):points(i)),x(3,points(i-1):points(i)),'linestyle',copt.plotunstable,'color',copt.plotcolor,'LineWidth',copt.plotlinewidth);
    else
        %draw full line
        line(x(1,points(i-1):points(i)),x(2,points(i-1):points(i)),x(3,points(i-1):points(i)),'linestyle',copt.plotstable,'color',copt.plotcolor,'LineWidth',copt.plotlinewidth);
    end
end

points = points(2:end-1);
line(x(1,points),x(2,cat(1,points)),x(3,points),'linestyle','none','marker',copt.plotmarker,'color',copt.plotcolormark); %*
d=axis;

skew = 0.03*[d(2)-d(1) d(4)-d(3) d(6)-d(5)];
labels = cat(1,s.label);
for i=2:length(s.index)-1;
text(x(1,s.index(i))+skew(1),x(2,s.index(i))+skew(2), x(3,s.index(i))+skew(3),labels(i));
end
hold off;


%SD:plots the 2d or 3d curve
